/*
 * Copyright 2024 The Error Prone Authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.google.errorprone.bugpatterns;

import com.google.errorprone.CompilationTestHelper;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.junit.runners.JUnit4;

@RunWith(JUnit4.class)
public class SystemConsoleNullTest {

  private final CompilationTestHelper testHelper =
      CompilationTestHelper.newInstance(SystemConsoleNull.class, getClass());

  @Test
  public void positive() {
    testHelper
        .addSourceLines(
            "Test.java", //
            "import java.io.Console;",
            "class Test {",
            "  boolean f() {",
            "    // BUG: Diagnostic contains:",
            "    return System.console() != null;",
            "  }",
            "  boolean g() {",
            "    // BUG: Diagnostic contains:",
            "    return System.console() == null;",
            "  }",
            "  boolean h() {",
            "    Console c = System.console();",
            "    // BUG: Diagnostic contains:",
            "    return c == null;",
            "  }",
            "  boolean i() {",
            "    Console c = System.console();",
            "    // BUG: Diagnostic contains:",
            "    return c != null;",
            "  }",
            "}")
        .doTest();
  }

  @Test
  public void negative() {
    testHelper
        .addSourceLines(
            "Test.java", //
            "import java.io.Console;",
            "class Test {",
            "  Console f() {",
            "    return System.console();",
            "  }",
            "  boolean h() {",
            "    Console c = System.console();",
            "    c = null;",
            "    return c == null;",
            "  }",
            "}")
        .doTest();
  }
}
